(* Exxx - For noForth FR59x9 mu.1, C&V version: Small robot
  Pulsewidth power control with 4KHz PWM at P3.6 & P3.7
  with a resolution of 1000 steps.

  P2.3 and P2.4 must be wired to the B-inputs of L9110 dual bridge
  driver. P3.6 and P3.7 must be wired to the A-inputs.

  Take care for the maximum USB-driver current of 150mA!

  Address 200 = P1IN,  port-1 input register
  Address 202 = P1OUT, port-1 output register
  Address 204 = P1DIR, port-1 direction register
  Address 206 = P1REN, port-1 resistor enable
  Address 20A = P1SEL0
  Address 20C = P1SEL1
  --
  Address 203 = P2OUT, port-2 output register
  Address 205 = P2DIR, port-2 direction register
  Address 20B = P2SEL0, port-2 selection-0 register
  Address 20D = P2SEL1, port-2 selection-1 register
  --
  Address 220 = P3IN,  port-3 input register
  Address 222 = P3OUT, port-3 output register
  Address 224 = P3DIR, port-3 direction register
  --
  Address 221 = P4IN,  port-4 input register
  Address 223 = P4OUT, port-4 output register
  Address 225 = P4DIR, port-4 direction register
  --
  Address 320 = PJIN,  port-J input register
  Address 322 = PJOUT, port-J output register
  Address 324 = PJDIR, port-J direction register
  --
  Address 340 = TA0CTL, timer A0 compare mode       RC servo
  Address 346 = TA0CCTL2, timer A0 output mode
  Address 352 = TA0CCR0, timer A0 period timing
  Address 356 = TA0CCR2, timer A0 Duty cycle
  --
  Address 380 = TA1CTL, timer A1 compare mode       MS timer
  Address 386 = TA1CCTL2, timer A1 output mode
  Address 392 = TA1CCR0, timer A1 period timing
  Address 396 = TA1CCR2, timer A1 Duty cycle
  --
  Address 400 = TA2CTL, timer A2 compare mode       US distance meter
  Address 402 = TA2CCTL0, timer A2 output mode
  Address 412 = TA2CCR0, timer A2 period timing
  Address 414 = TA2CCR1, timer A2 Duty cycle
  --
  Address 440 = TA3CTL, timer A3 compare mode       Free
  Address 442 = TA3CCTL0, timer A3 output mode
  Address 452 = TA3CCR0, timer A3 period timing
  Address 454 = TA3CCR1, timer A3 Duty cycle
  --
  Address 3C0 = TB0CTL, timer B0 compare mode       PWM Motors
  Address 3C6 = TB0CCTL2, timer B0 output mode
  Address 3D2 = TB0CCR0, timer B0 period timing
  Address 3D6 = TB0CCR2, timer B0 Duty cycle
 --
  Address 800 = ADC12CTL0
  Address 802 = ADC12CTL1
  Address 804 = ADC12CTL2
  Address 806 = ADC12CTL3
  Address 860 = ADC12MEM0

  FEDCBA9876543210 bit-numbers
  0000000000010000 - Choose output bit4 or bit5
  0000000011000000 - toggle-set output
  0000001000110100 - TA clear, up/down, SMCLK, no presc.

  Use a low power version of noForth to explore this code maximal!
  At least implement the low power MS version! At 1 MHz this uses
  about 70uA only! This gives the LiPo maximal protection.
  A mosfet to switch off power users fully is recommended too.

  C0 200 - P1IN   Input bits
  C0 202 - P1OUT  Output bits
  C0 204 - P1DIR  Direction bits

  * Routines for reflection sensors ...... Ok
  * Routines for contact sensor .......... Ok
  * Mounting of motors ................... ?
  * Adding 2n2 to reset .................. Ok
  * Adding 470 Ohm in gate of SI2302 ..... Ok
  * Testing Mushi sensor test ............ Ok
  * Add ultrasonic sensor RCW-0001 ....... Ok
  * Add LiPo voltage to OLED display ..... Ok

 *)

hex
inside also definitions
dm 1000 constant #CYCLUS        \ 4 kHz PWM range 1000 steps
value POWER

: >LEFT     ( 0 to #cyclus -- )
    #cyclus umin  3DE !         \ TB0CCR6   Set pulselength left motor
    080 222 *bic ;              \ P3OUT

: >RIGHT    ( 0 to #cyclus -- )
    #cyclus umin  3DC !         \ TB0CCR5   Set pulselength right motor
    040 222 *bic ;              \ P3OUT

: >PWM      ( +n -- )   dup >right  >left ;         \ Pulselength for both
: FW        ( -- )      18 203 *bic  power >pwm ;   \ Move forward primitive
forth definitions
: BACKW     ( -- )      18 203 *bis  #cyclus 1+ power - >pwm ; \ Move backward
: STOP      ( -- )      18 203 *bic  0 >pwm ;       \ Stop moving
: FORW      ( -- )      backw  dm 60 ms  fw ;       \ Move forward
: LEFT      ( -- )      forw  1 >left ;             \ Stop left motor
: RIGHT     ( -- )      forw  1 >right ;            \ Stop right motor

inside definitions
\ Adjust speed in correct direction, but only when not stopped
: SPEED)    ( -- )
    3DE @ if  18 203 bit* if  backw exit  then  fw  then ;

forth definitions
: SPEED     ( +n -- )   dm 800 umin  dm 200 +  to power  speed) ; \ Set speed
: .SPEED    ( -- )      power dm 200 - . ;      \ Show current speed
inside definitions
: PWM-OFF   ( -- )      0 3C0 !  0C0 22A *bic ; \ TB0CTL, P2SEL  Deactivate PWM

\ PWM at P2.3 & P2.4, stop motors
: PWM-ON        ( -- )
    0C0 22A *bis 0C0 224 *bis   \ P3SEL0, P3DIR  Set PWM to output P3.6&7
    018 205 *bis 018 203 *bic   \ P2DIR, P2OUT   P2.3 & P2.4 low
    0 3C0 !  #cyclus 3D2 !      \ TB0CTL, TB0CCR0  Set period time
    040 3CE !                   \ TB0CCTL6 Set output mode positive pulse
    040 3CC !                   \ TB0CCTL5 Set output mode positive pulse (C0)
    274  3C0 !                  \ TB0CTL   Activate timer
    dm 200 speed  stop ;        \ Set basic speed, but stop motors

pwm-off  fresh  definitions
shield MOTORS\  freeze


hex  inside also  definitions
code INT-ON     ( -- )      #8 sr bis  next  end-code
code INT-OFF    ( -- )      #8 sr bic  next  end-code

\ TA1 for MS-timer
: MS-ON     ( -- )          \ Initialise pulse hardware
    int-off                 \ Deactivate
    0 380 !                 \ TA1CTL   Stop timer-A1
    DM 2000 392 !           \ TA1CCR0  Interrupt after every millisec.
    0 390 !                 \ TA1R     Start at zero
\   0010 382 !              \ TA1CCTL0 Set compare 0 interrupt on
    02D4  380 !             \ TA1CTL   Start timer in up mode /8
    int-on ;                \ Activate

code LPM0     ( -- )    18 # sr bis  next  end-code \ To LPM0 & intrpt on
code LPM1     ( -- )    58 # sr bis  next  end-code \ To LPM1 & intrpt on
code LPM4     ( -- )    F8 # sr bis  next  end-code \ To LPM4 & intrpt on

routine COUNT-MS            \ 6 -   Interrupt call
    #0 adr ms) & cmp        \ 5 - 4 Decrease MS counter
    0=? if,                 \ 2 - 2
        10 # 382 & bic      \ 6 - 6 Disable compare 0 interrupt
        F0 # rp ) bic       \ 2 - 4 To All Mode, LPM off
        reti                \ 5 - 2
    then,
    #1 adr ms) & sub        \ 5 - 4 TA0CCR0 Set next period
    reti                    \ 5 - 2
end-code

forth definitions
: MS         ( u -- )    to ms)  10 382 **bis  lpm1 ;

count-ms   FFE2 vec!        \ Install Timer-A1 vector for MS
fresh definitions
shield MS\  freeze


\ Reading of a reflection sensor type TCRT5000
\ Using 3V3 to 3V6 and 220 Ohm in series with led.
\ 12 kOhm from collector to +3V3 to +3V6 the collector
\ is connected to an analog input from the MSP430
hex
: >LEDS  ( b -- )  322 c! ; \ PJOUT

(* ADC on, sample time at 64 clocks
   ADC-clock/4, sample timer on, 12-bit resolution
    P1.0 = A0   = 00
    P1.1 = A1   = 01
    P1.2 = A2   = 02
    P1.4 = A4   = 04 ( Accu )
    P1.5 = A5   = 05
    P3.1 = A13  = 0D
    P3.2 = A14  = 0E

  12-bit ADC using internal ADC12 of MSP430FR5xxx
*)

inside also  definitions

(* Accu must use 2.5 Volt reference!
   REFCTL0 = 01B0 = 0031 select 2.5 Volt reference
*)

: REF25         31 1B0 ! ;  \ Activate VREF at 2.5 Volt
: REFOFF        01 1B0 **bic ; \ Deactivate REF

: ADC-ON    ( -- )
    02 800 **bic            \ ADC12CTL0  Clear ENC
    37 20A *bis             \ P1SEL0     P1.4 = ADC in  (37 see below)
    37 20C *bis             \ P1SEL1     (37=P1.0, P1.1, P1.2, P1.4 & P1.5)
    06 22A *bis             \ P3SEL0     (P3.1, P3.2)
    06 22C *bis             \ P3SEL1
    0410 800 !              \ ADC12CTL0  Sampletime 64 clocks, ADC on
    2200 802 !              \ ADC12CTL1  Clock/4, use sample timer
    0020 804 !              \ ADC12CTL2  12 bit resolution
    02 800 **bis ;          \ ADC12CTL0  Set ENC

: ADC-OFF   ( -- )      02 800 **bic  10 800 **bic  02 800 **bis ;

forth definitions
\ We need to clear the ENC bit before setting a new input channel
\ ADC can be used with VCC and VREF (2.5V) as reference voltage
: ADC       ( +n fl -- u )
    02 800 **bic            \ ADC12CTL0  Clear ENC
    100 and >r              \ Use VREF when fl is true
    1F and  r> or 820 !     \ ADC12MCTL0 Select input
    03 800 **bis            \ ADC12CTL0  Set ENC & ADC12SC
    begin 1 802 bit** 0= until \ ADC12CTL1  ADC12 busy?
    860 @ ;                 \ ADC12MEM0  Read result

: ACCU          04 -1 adc ; \ Read accu level at P1.4 using VREF
: REFLECTION    0 adc ;     \ Read sensor level at Ax using VCC

(* Test routines for ADC

: SENS      ( -- )
    adc-on  decimal          \ Initialise
    begin
        00 reflection 38 /   \ Read TCRT5000 and scale
        3F umin dup . >leds  \ Print visual range on led array
    key? until 0 >leds hex ; \ Leds off

value mi  value ma
: SM    0 to ma   5000 to mi  adc-on ;
: MM    00 reflection  dup mi umin to mi  dup ma umax to ma . ;
: .M    mi . ma . ;
*)

adc-on  fresh definitions
shield SENSORS\  freeze


\ Switch Muchi mostly off when LiPo is almost empty
\ Using a low power version of MS even more power is saved.
\ First change DCO frequency between 1MHz and 16MHz

hex
\ Set DCO to 1 MHz, current use of CPU on, power-off state AM~0,4mA
code 1MHZ       ( -- )
    #8 sr bic               \ Interrupts off
    A5 # 161 & .b mov       \ CSCTL0h Open DCO change
    0040 # 162 & mov        \ CSCTL1  DCO = 1MHz
    #0 161 & .b mov         \ CSCTL0h Freeze DCO
\ Configure eUSCI A1 as UART
    0081 # 5E0 & mov        \ UCA0CTLW0  Select clock source, USCI off
    DM 06 # 5E6 & mov       \ UCA1BRW    Set baudrate 9600 at 1MHz
    2081 # 5E8 & mov        \ UCA1CTLW   Set modulation control
    #1 5E0 & bic            \ USCI active
\   03A # adr ms) & mov     \ MS timing for 1 MHz DCO
    #0 380 & mov            \ Stop timer-A1
    dm 1000 # 392 & mov     \ Delay 1 ms
    214 # 380 & mov         \ Start timer-A1 /1, SMCLK
    #8 sr bis               \ Interrupts on
    next
end-code

\ Set DCO to 16 MHz, current use of CPU on, power-off state AM~1,3mA
code 16MHZ      ( -- )
    #8 sr bic               \ Interrupts off
    A5 # 161 & .b mov       \ CSCTL0h Open DCO change
    0048 # 162 & mov        \ CSCTL1  DCO = 1MHz
    #0 161 & .b mov         \ CSCTL0h Freeze DCO
\ Configure eUSCI A1 as UART
    0081 # 5E0 & mov        \ UCA0CTLW0  Select clock source, USCI off
    DM 104 # 5E6 & mov      \ UCA1BRW    Set baudrate 9600 at 16MHz
    D621 # 5E8 & mov        \ UCA1CTLW   Set modulation control
    #1 5E0 & bic            \ USCI active
\   250 # adr ms) & mov     \ MS timing for 16 MHz DCO
    #0 380 & mov            \ Stop timer-A1
    dm 2000 # 392 & mov     \ Delay 1 ms
    2D4 # 380 & mov         \ Start timer-A1 /8, SMCLK
    #8 sr bis               \ Interrupts on
    next
end-code

(* Accu empty test
r20= 100 kOhm           R20=47kOhm
4.4 Volt ADC = 4095        4095
4.3 Volt ADC = 4095        4050
4.2 Volt ADC = 4095        3970
4.1 Volt ADC = 4095        3852
4.0 Volt ADC = 4095        3750
3.5 Volt ADC = 4085        3260
3.4 Volt ADC = 3965        3140
3.3 Volt ADC = 3840        3100
3.2 Volt ADC = 3725        2980
3.1 Volt ADC = 3600        2860
3.0 Volt ADC = 3480        2760
2.9 Volt ADC = 3353        2660
2.8 Volt ADC = 3240        2610
2.7 Volt ADC = 3113        ..
2.6 Volt ADC = 3055        ..
*)

inside also
: LOWPOWER  ( -- )      \ Most I/O to output
    D8 204 *bis  D8 202 *bis    \ P1
    FF 205 *bis  FF 203 *bis    \ P2
    F9 224 *bis  D9 222 !       \ P3  F9
    FF 225 *bis  FF 223 *bis    \ P4
    FF 324 *bis  EF 322 *bis    \ PJ  FF
    pwm-off  1mhz ;             \ Stop motors, DCO = 1MHz

: NORMALPOWER ( -- )
    08 204 !     00 202 !       \ P1
    1E 205 !     80 203 !       \ P2
    E9 224 !     30 222 !       \ P3
    00 225 !     10 223 !       \ P4
    3F 324 !     00 322 !       \ PJ
    pwm-on  16mhz ;             \ Motors on, DCO  16 MHz

\ Main power-up/down bit to a mosfet switch to power down Mushi
\ After this only the MSP430, the MCP131 and the 3V3 stabiliser
\ are getting powered
\ : SETUP-PW  ( -- )     20 224 *bis ;   \ P3DIR  Bit-5 is output
\ : POWER-ON  ( -- )     20 222 *bis ;   \ P3OUT  Power up Mushi
\ : POWER-OFF ( -- )     20 222 *bic ;   \ P3OUT  Power down Mushi

\ Average curent while waiting for power is .258 to .9 mA
\ Next step, add a low power MS routine.
: LIPO-EMPTY  ( -- )        \ Short flashing of led when accu is empty
    adc-on  ref25  ms-on
    accu dm 3100 < if       \ Accu almost empty (lower then 3V3)?
        lowpower            \ Switch Mushi to low energy usage
        begin
            FF >leds  1 ms  \ Flash led
            EF >leds  200 ms
        accu dm 3750 > until \ Do until LiPo is above 75% (4 Volt)
        normalpower         \ Mushi to normal modus
    then ;                  \ Back to full energy

: @ACCU       ( -- Volt )   \ Scale accu reading to fixed point voltage
    accu 10 8F */ ;

: .ACCU      ( -- )         \ Show LiPo voltage in fixed point
    @accu 0 <# # # ch . hold # #> type space ;

shield LIPO\  freeze


inside also  definitions
hex             \ 50 Hz PWM for controlling servos
value PW        \ Space for PWM value
value STATUS    \ Interrupt status

forth definitions
\ Set servo position in steps from 0 to 255
: PULSE     ( u -- )    0C * DM 1585 + DM 4550 umin  to pw ;

inside definitions
\ This interrupt gives .78 to 2.25 millisec. pulses at 50 Hz (24 cells, 46 cycles)
\ routine PULSE-INT
\    9382 ,  adr status ,  2009 ,  4292 ,  adr pw ,
\    352 ,  D0F2 ,  1 , 222 ,  5392 ,  adr status ,
\    1300 ,  40B2 ,  8CA0 ,  352 ,  C0F2 ,
\    1 ,  222 ,  4382 ,  adr status ,  1300 ,
\ end-code
routine PULSE-INT           \ 6 - interrupt call
    #0 adr status & cmp     \ 1 - 2 First the servo pulse?
    =? if,                  \ 2 - 1 Yes
        adr pw & 352 & mov  \ 5 - 3 TA0CCR0 Set next period
        #1 222 & .b bis     \ 5 - 3 P3OUT  Set servo bit on (P3.0)
        #1 adr status & add \ 1 - 2 Set bit pointer on de pause position
        reti                \ 5 - 1
    then,
    dm 36000 # 352 & mov    \ 5 - 3 TA0CCR0 Set pause period
    #1 222 & .b bic         \ 5 - 3 P3OUT  Reset servo bit (P3.0)
    #0 adr status & mov     \ 5 - 2
    reti                    \ 5 - 1
end-code

\ SERVO at P3.0 etc.
: PULSE-ON  ( -- )          \ Initialise pulse hardware
    001 224 *bis            \ P3DIR    Bit P3.0 is output
    0 340 !                 \ TA0CTL   Stop timer-A0
    DM 2000 352 !           \ TA0CCR0  First interrupt after 1 ms
    02D4  340 !             \ TA0CTL   Start timer
    0010 342 !              \ TA0CCTL0 Set compare 0 interrupt on
    0 to status             \ Start with pulse
    DM 125 pulse            \ Default pulse lenght is 1,5 ms
    int-on ;                \ Activate

(* Servo test routines
: UP        ( -- )          \ Pulse longer
    0  begin
        dup pulse  0A ms  1+
    dup 0FF >  key?  or until  drop ;

: DOWN      ( -- )          \ Pulse shorter
    FF  begin
        dup pulse  0A ms  1-
    dup 0<  key?  or until  drop ;

: SERVO     ( -- )          \ Demonstrate (micro) servo at P3.0
    pulse-on  begin  up  down  key? until ;


\ A extra servo tester in just a few lines
: .PULSE    ( -- )          \ Show servo position in microseconds
    s? 0= if                \ S2 pressed?
      cr  pw u.             \ Yes, show microsecond value
        begin  s? until     \ S2 released?, Yes, go on
        50 ms  int-on
    then ;

\ Set servo position in steps from 0 to 4000
: TPULSE    ( u -- )    DM 1200 + DM 5200 umin  to pw ;

\ A servo tester using the sensor at P1.1 and the servo at P3.0
\ When S2 is pressed the current servo position is printed.
: TESTER    ( -- )
    adc-on  pulse-on  decimal
    begin  1 0 adc tpulse  .pulse
    key? until  hex ;
*)


(* US distance meter with RCW-0001
  Register addresses
320 = PJIN      - Input register
322 = PJOUT     - Output register
324 = PJDIR     - Direction register
326 = PJREN     - Resistance on/off

Addresses for timer-A2 and timer-A3, when TA2 is already used
and TA3 is free just change the numbers in the source!
Replace 160 by 180 and replace 170 by 190, ready.
400 = TA2CTL   - Timer A2 control
410 = TA2R     - Timer A2 register
440 = TA3CTL   - Timer A3 control
450 = TA3R     - Timer A3 register
00000010.11100100 = 02E4 - TA is nul, overflow mode, SMCLK, presc. /8

For all US-sensors:
  Echo = PJ.0
  Trig = PJ.1

The protocol for this sensor is:
1- Give trigger pulse of at least 10us at 'Trig'.
2- Wait for 'Echo' to go high
3- Wait for 'Echo' to go low while counting the pulselength
4- Convert the resulting number to millimeter or centimeter

This example is timed by timer-A2 with a microsecond resolution.
When timer-A2 is allready used, just change the numbers as explaned below!

Note that: activated interrupts will not influence the result.
This variant will never hang, it times out after about 65 millisec.
*)

hex  inside also  definitions
: FLASH     ( -- )          10 322 *bis 200 ms  10 322 *bic 200 ms ;

: US-ON     ( -- )
    01 324 *bic     \ PJDIR  PJ.0 Input with pullup
    01 326 *bis     \ PJREN
    01 322 *bis     \ PJOUT
    02 324 *bis     \ PJDIR  PJ.1 Output
    02 322 *bic ;   \ PJOUT

: TRIGGER   ( -- )      02 322 *bis  noop noop noop  02 322 *bic ; \ PJOUT
: START     ( -- )      2E4 400 !  0 410 ! ; \ TA2CTL, TA2R  Start counter
: STOP      ( -- )      030 400 *bic ; \ TA2CTL  Stop counter

: DISTANCE) ( -- -1|microsec )
    trigger                         \ Activate sensor
    begin  01 320 bit* until        \ PJIN   Wait for start of echo pulse
    start                           \ Start counter
    begin
        1 400 bit* if -1 exit then  \ TA2CTL Counter overflow then ready!
    01 320 bit* 0= until            \ PJIN   Wait for echo to end
    stop  410 @ 2/ ( Pulse len. ) ; \ TA2R   Stop & read counter

forth definitions
: DISTANCE  ( -- distance in cm )   distance) 5 123 */ ;

: MEASURE   ( -- )              \ Show distance in steps of 2 cm
    us-on  flash  base @ >r  decimal
    begin  distance 3 .r space  40 ms  key? until
    r> base ! ;

fresh definitions
shield US\  freeze


(* Add floor sensor routine
    P1.0 = A0   = 00  Right front
    P1.1 = A1   = 01  Right line
    P1.2 = A2   = 02  Middle line
    P1.5 = A5   = 05  Left line
    P3.1 = A13  = 0D  Left front
    P3.2 = A14  = 0E  Middle back
*)

forth definitions
: FLOOR     (  adc -- s )       reflection 400 / ;
: FBACK     ( -- s )            0E reflection 148 / ;


(* Add contact sensor routines
    P2.7 = 80 0201  = Back feeler
    P3.4 = 10 0220  = Left feeler
    P4.4 = 10 0221  = Right feeler
*)

: LCONTACT  ( -- flag )     10 220 bit* 0= 0= ; \ P3.4
: RCONTACT  ( -- flag )     10 221 bit* 0= 0= ; \ P4.4
: BCONTACT  ( -- flag )     80 201 bit* 0= 0= ; \ P2.7


(* Control the leds on the back
    P2.2 = 04 203 = LB0
    P3.3 = 08 222 = LB1
    P2.1 = 02 203 = LB2
    P1.3 = 08 202 = LB4
*)

inside also  definitions
: LB0       ( fl -- )   if  04 203 *bis  else  04 203 *bic  then ;
: LB1       ( fl -- )   if  08 222 *bis  else  08 222 *bic  then ;
: LB2       ( fl -- )   if  02 203 *bis  else  02 203 *bic  then ;
: LB3       ( fl -- )   if  08 202 *bis  else  08 202 *bic  then ;

forth definitions
: >BACK     ( +n -- )       \ Set low nibble to leds on the back
    dup 1 and lb0  dup 2 and lb1
    dup 4 and lb2  8 and lb3 ;

: KNIGHT    ( u -- )        \ Knight rider on the back
    0 ?do
        1  4 0 do  dup >back  2*  50 ms  loop
        4 0 do  2/  dup >back  50 ms  loop  drop
    loop
    0 >back ;

inside definitions
: MUSHI-SETUP ( -- )        \ Setup all Mushi hardware
    adc-on   ref25          \ Init ADC & reference
    pwm-on                  \ Start PWM
    pulse-on                \ Start servo pulses
    ms-on  us-on            \ Start low power MS & US-distance meter
    3 knight ;              \ Show knightrider at startup


' mushi-setup  to app       \ Initialise all Mushi hardware
pulse-int  FFEA vec!        \ Install Timer-A0 vector for servo pulses
fresh definitions
shield PULSE\  freeze


(* Extra experimental code

: INIRESET  ( -- )      1C 104 ! ;  \ Does not improve glitches

*)

                              ( End )
