hex
(* ADC on, sample time at 64 clocks
   ADC-clock/4, sample timer on, 12-bit resolution
    P1.0 = A0   = 00
    P1.1 = A1   = 01
    P1.2 = A2   = 02
    P1.4 = A4   = 04 ( Accu )
    P1.5 = A5   = 05
    P3.1 = A13  = 0D
    P3.2 = A14  = 0E

    12-bit ADC using internal ADC12 of MSP430FR5xxx

    Accu must use 2.5 Volt reference!
    REFCTL0 = 01B0 = 0031 select 2.5 Volt reference
*)

inside also  definitions
: REF25         31 1B0 ! ;  \ Activate VREF at 2.5 Volt
: REFOFF        01 1B0 **bic ; \ Deactivate REF

: ADC-ON    ( -- )
    02 800 **bic            \ ADC12CTL0  Clear ENC
    37 20A *bis             \ P1SEL0     P1.4 = ADC in  (37 see below)
    37 20C *bis             \ P1SEL1     (37=P1.0, P1.1, P1.2, P1.4 & P1.5)
    06 22A *bis             \ P3SEL0     (P3.1, P3.2)
    06 22C *bis             \ P3SEL1
    0410 800 !              \ ADC12CTL0  Sampletime 64 clocks, ADC on
    2200 802 !              \ ADC12CTL1  Clock/4, use sample timer
    0020 804 !              \ ADC12CTL2  12 bit resolution
    02 800 **bis ;          \ ADC12CTL0  Set ENC

forth definitions
\ We need to clear the ENC bit before setting a new input channel
\ ADC can be used with VCC and VREF (2.5V) as reference voltage
: ADC       ( +n fl -- u )
    02 800 **bic            \ ADC12CTL0  Clear ENC
    100 and >r              \ Use VREF when fl is true
    1F and  r> or 820 !     \ ADC12MCTL0 Select input
    03 800 **bis            \ ADC12CTL0  Set ENC & ADC12SC
    begin 1 802 bit** 0= until \ ADC12CTL1  ADC12 busy?
    860 @ ;                 \ ADC12MEM0  Read result

: ACCU          04 -1 adc ; \ Read accu level at P1.4 using VREF
: REFLECTION    0 adc ;     \ Read sensor level at Ax using VCC

: FLOOR     (  adc -- s )       reflection 400 / ;
: FBACK     ( -- s )            0E reflection 148 / ;

: @ACCU     ( -- Volt )      \ Scale accu reading to fixed point voltage
    accu 10 8F */ ;

: .ACCU     ( -- )           \ Show LiPo voltage in fixed point
    @accu 0 <# # # ch . hold # #> type space ;


\ Switch Muchi mostly off when LiPo is almost empty
\ Using a low power version of MS even more power is saved.
\ First change DCO frequency between 1MHz and 16MHz

\ : POWER-ON  ( -- )     20 222 *bis ;   \ P3OUT  Power up Mushi
\ : POWER-OFF ( -- )     20 222 *bic ;   \ P3OUT  Power down Mushi

inside also
: LOWPOWER  ( -- )      \ Most I/O to output
    D8 204 *bis  D8 202 *bis    \ P1
    FF 205 *bis  FF 203 *bis    \ P2
    F9 224 *bis  D9 222 !       \ P3  F9
    FF 225 *bis  FF 223 *bis    \ P4
    FF 324 *bis  EF 322 *bis    \ PJ  FF
    pwm-off  1mhz ;             \ Stop motors, DCO = 1MHz

: NORMALPOWER ( -- )
    08 204 !     00 202 !       \ P1
    1E 205 !     80 203 !       \ P2
    E9 224 !     30 222 !       \ P3
    00 225 !     10 223 !       \ P4
    3F 324 !     00 322 !       \ PJ
    pwm-on  16mhz ;             \ Motors on, DCO  16 MHz

\ Average curent while waiting for power is .258 to .9 mA
\ Next step, add a low power MS routine.
: LIPO-EMPTY  ( -- )        \ Short flashing of led when accu is empty
    accu dm 3100 < if       \ Accu almost empty (lower then 3V3)?
        lowpower            \ Switch Mushi to low energy usage
        begin
            FF >leds  1 ms  \ Flash led
            EF >leds  200 ms
        again               \ Do until Mushi is switched off
    then ;

shield LIPO\  freeze

                              ( End )
